from typing import List
from dataclasses import dataclass


@dataclass
class FewShotExampleInputParam:
    name: str
    value: str

    def __str__(self) -> str:
        return f"{self.name}: {self.value}"


class FewShotExample:
    """
    Class representing an example of the evaluation that could be used for few-shot prompting.
    """

    # Name of the evaluation function
    input_params: List[FewShotExampleInputParam]
    # Evaluation result - Pass or Fail
    eval_result: str
    # LLM's reason for evaluation
    eval_reason: str

    def __init__(
        self,
        input_params: List[FewShotExampleInputParam],
        eval_result: str,
        eval_reason: str,
    ):
        """
        Initialize a new instance of FewShotExample.
        """
        self.input_params = input_params
        self.eval_result = eval_result
        self.eval_reason = eval_reason

    def __str__(self):
        """
        Return a string representation of the FewShotExample.
        """

        input_params_str = "\n".join([str(param) for param in self.input_params])

        return (
            f"{input_params_str} \n"
            + f"result: {self.eval_result} \n"
            + f"reason:{self.eval_reason} \n"
        )



from athina.evals import Regex, ContainsAny, ContainsAll, Contains, ContainsNone, ContainsJson, ContainsEmail, IsJson, IsEmail, NoInvalidLinks, ContainsLink, ContainsValidLink, Equals, StartsWith, EndsWith, LengthLessThan, LengthGreaterThan, LengthBetween, ApiCall, DoesResponseAnswerQuery, Faithfulness, BaseEvaluator, ContextContainsEnoughInformation, SummaryAccuracy, Groundedness, GradingCriteria, CustomPrompt, RagasContextRelevancy, RagasAnswerRelevancy, RagasAnswerCorrectness, RagasAnswerSemanticSimilarity, RagasCoherence, RagasConciseness, RagasContextPrecision, RagasContextRecall, RagasFaithfulness, RagasHarmfulness, RagasMaliciousness, NotGibberishText, SafeForWorkText, ContainsNoSensitiveTopics, OpenAiContentModeration, PiiDetection, PromptInjection, ProfanityFree, ReadingTime, DetectPII, ToxicLanguage, CorrectLanguage, NoSecretsPresent, RestrictToTopic, NotUnusualPrompt, PolitenessCheck, OneLine, JsonEval, ConversationResolution, ConversationCoherence
from athina.evals.grounded.similarity import CosineSimilarity, JaccardSimilarity, JaroWincklerSimilarity, NormalisedLevenshteinSimilarity, SorensenDiceSimilarity
from athina.evals.grounded.wrapper import AnswerSimilarity, ContextSimilarity

grounded_operations = {
    "AnswerSimilarity": AnswerSimilarity,
    "ContextSimilarity": ContextSimilarity,
}

conversation_operations = {
    "ConversationResolution": ConversationResolution,
    "ConversationCoherence": ConversationCoherence
}

function_operations = {
    "Regex": Regex,
    "ContainsAny": ContainsAny,
    "ContainsAll": ContainsAll,
    "Contains": Contains,
    "ContainsNone": ContainsNone,
    "ContainsJson": ContainsJson,
    "ContainsEmail": ContainsEmail,
    "IsJson": IsJson,
    "IsEmail": IsEmail,
    "NoInvalidLinks": NoInvalidLinks,
    "ContainsLink": ContainsLink,
    "ContainsValidLink": ContainsValidLink,
    "Equals": Equals,
    "StartsWith": StartsWith,
    "EndsWith": EndsWith,
    "LengthLessThan": LengthLessThan,
    "LengthGreaterThan": LengthGreaterThan,
    "LengthBetween": LengthBetween,
    "ApiCall": ApiCall,
    "OneLine": OneLine,
    "JsonEval": JsonEval
}

safety_operations = {
    "SafeForWorkText": SafeForWorkText,
    "NotGibberishText": NotGibberishText,
    "ContainsNoSensitiveTopics": ContainsNoSensitiveTopics,
    "OpenAiContentModeration" : OpenAiContentModeration,
    "PiiDetection": PiiDetection,
    "PromptInjection": PromptInjection,
    "ProfanityFree": ProfanityFree,
    "ReadingTime": ReadingTime,
    "DetectPII": DetectPII,
    "ToxicLanguage": ToxicLanguage,
    "CorrectLanguage": CorrectLanguage,
    "NoSecretsPresent": NoSecretsPresent,
    "RestrictToTopic": RestrictToTopic,
    "NotUnusualPrompt": NotUnusualPrompt,
    "PolitenessCheck": PolitenessCheck
}

llm_operations = {
    "Draq": DoesResponseAnswerQuery,
    "Irftc": Faithfulness,
    "BaseEvaluator": BaseEvaluator,
    "Ccei": ContextContainsEnoughInformation,
    "SummarizationHal": SummaryAccuracy,
    "Groundedness": Groundedness,
    "GradingCriteria": GradingCriteria,
    "CustomPrompt": CustomPrompt,
}

ragas_operations = {
    "RagasContextRelevancy": RagasContextRelevancy,
    "RagasAnswerRelevancy": RagasAnswerRelevancy,
    "RagasAnswerCorrectness": RagasAnswerCorrectness,
    "RagasAnswerSemanticSimilarity": RagasAnswerSemanticSimilarity,
    "RagasCoherence": RagasCoherence,
    "RagasConciseness": RagasConciseness,
    "RagasContextPrecision": RagasContextPrecision,
    "RagasContextRecall": RagasContextRecall,
    "RagasFaithfulness": RagasFaithfulness,
    "RagasHarmfulness": RagasHarmfulness,
    "RagasMaliciousness": RagasMaliciousness
}

def get_evaluator(evaluator_type):
    if evaluator_type in function_operations:
        return function_operations[evaluator_type]
    elif evaluator_type in safety_operations:
        return safety_operations[evaluator_type]
    elif evaluator_type in grounded_operations:
        return grounded_operations[evaluator_type]
    elif evaluator_type in llm_operations:
        return llm_operations[evaluator_type]
    elif evaluator_type in ragas_operations:
        return ragas_operations[evaluator_type]
    elif evaluator_type in conversation_operations:
        return conversation_operations[evaluator_type]
    else:
        raise ValueError(f"Invalid evaluator type: {evaluator_type}")

# TODO : Remove the following methods from workers repo to reduce code duplication
def get_comparator(comparator_name):
    if comparator_name is None:
        raise ValueError("similarity_function is a required argument")
    comparators = {
        "CosineSimilarity": CosineSimilarity(),
        "NormalisedLevenshteinSimilarity": NormalisedLevenshteinSimilarity(),
        "JaroWincklerSimilarity": JaroWincklerSimilarity(),
        "JaccardSimilarity": JaccardSimilarity(),
        "SorensenDiceSimilarity": SorensenDiceSimilarity()
    }
    comparator = comparators.get(comparator_name, None)
    if comparator is None:
        raise NotImplementedError(f"Comparator {comparator_name} not implemented.")
    return comparator

def create_grounded_evaluator(grounded_eval_name, comparator, failure_threshold):
    grounded_evaluator_class = grounded_operations.get(grounded_eval_name, None)
    if grounded_evaluator_class is None:
        raise NotImplementedError(f"Grounded eval {grounded_eval_name} not implemented.")
    else:
        return grounded_evaluator_class(comparator=comparator, failure_threshold=failure_threshold)



import time
from typing import List
from athina.interfaces.athina import (
    AthinaEvalRequestCreateRequest,
    AthinaEvalRequestSource,
    AthinaEvalResult,
    AthinaJobType,
    AthinaEvalRunResult,
    AthinaInterfaceHelper,
)
from athina.interfaces.result import EvalResult
from athina.services.athina_api_service import AthinaApiService
from athina.keys import AthinaApiKey
from athina.constants.messages import AthinaMessages

class AthinaLoggingHelper:
    @staticmethod
    def log_eval_performance_report(*args, **kwargs):
        """
        Passthrough method: Checks if the user has set an Athina API key
        """
        if AthinaApiKey.is_set():
            return AthinaApiService.log_eval_performance_report(*args, **kwargs)
        
    @staticmethod
    def log_experiment(*args, **kwargs):
        """
        Passthrough method: Checks if the user has set an Athina API key
        """
        if AthinaApiKey.is_set():
            return AthinaApiService.log_experiment(*args, **kwargs)

    @staticmethod
    def create_eval_request(eval_name: str, request_data: dict, request_type: str):
        try:
            if not AthinaApiKey.is_set():
                return None
            # Create eval request
            eval_request = AthinaEvalRequestCreateRequest(
                request_label=eval_name + "_eval_" + str(time.time()),
                request_data=request_data,
                request_data_type=request_type,
                source=AthinaEvalRequestSource.DEV_SDK.value,
            )
            eval_request_id = AthinaApiService.create_eval_request(eval_request)[
                "data"
            ]["eval_request"]["id"]
            return eval_request_id
        except Exception as e:
            print(
                f"An error occurred while creating eval request",
                str(e),
            )
            raise

    @staticmethod
    def log_eval_results(
        eval_request_id: str,
        eval_results: List[EvalResult],
    ):
        try:
            if not AthinaApiKey.is_set():
                return
            athina_eval_result_create_many_request = []

            for eval_result in eval_results:
                # Construct eval result object
                failed_percent = float(eval_result.get("failure")) if "failure" in eval_result else None
                metrics = eval_result.get("metrics", [])
                datapoint_field_annotations = eval_result.get("datapoint_field_annotations", None)
                athina_eval_result = AthinaEvalResult(
                    job_type=AthinaJobType.LLM_EVAL.value,
                    failed_percent=failed_percent,
                    number_of_runs=1,
                    flakiness=0.0,
                    run_results=[
                        AthinaEvalRunResult(
                            failed=eval_result["failure"] if "failure" in eval_result else None,
                            runtime=eval_result["runtime"],
                            reason=eval_result["reason"],
                            datapoint_field_annotations=datapoint_field_annotations,
                        )
                    ],
                    data=eval_result["data"],
                    runtime=eval_result["runtime"],
                    metrics=metrics,
                    display_name=eval_result["display_name"],
                )

                # log eval results to Athina
                athina_eval_result_create_request = (
                    AthinaInterfaceHelper.eval_result_to_create_request(
                        eval_request_id=eval_request_id,
                        eval_type=eval_result["name"],
                        language_model_id=eval_result["model"] if "model" in eval_result else None,
                        eval_result=athina_eval_result,
                    )
                )
                athina_eval_result_create_request_dict = {
                    k: v
                    for k, v in athina_eval_result_create_request.items()
                    if v is not None
                }
                athina_eval_result_create_many_request.append(
                    athina_eval_result_create_request_dict
                )
            AthinaApiService.log_eval_results(athina_eval_result_create_many_request)

        except Exception as e:
            print(
                f"An error occurred while posting eval results",
                str(e),
            )
            raise

    @staticmethod
    def log_eval_results_with_config(eval_results_with_config: dict, dataset_id: str):
        try:
            def remove_none_values(data: dict) -> dict:
                return {k: v for k, v in data.items() if v is not None}

            eval_results = eval_results_with_config.get("eval_results", [])
            # Limit to the first 1000 items
            sliced_eval_results = eval_results[:1000]
            cleaned_eval_results = []

            for eval_result in sliced_eval_results:
                cleaned_eval_result = {
                    "metrics": eval_result.get("metrics"),
                    "reason": eval_result.get("reason")
                }
                cleaned_eval_results.append(remove_none_values(cleaned_eval_result))

            development_eval_config = remove_none_values(eval_results_with_config.get("development_eval_config", {}))

            cleaned_results = {
                "dataset_id": dataset_id,
                "eval_results": cleaned_eval_results,
                "development_eval_config": development_eval_config
            }

            # Replace with your logging mechanism
            AthinaApiService.log_eval_results_with_config(cleaned_results)
        except Exception as e:
            raise



from athina.interfaces.model import Model
from ..ragas_evaluator import RagasEvaluator
from athina.evals.eval_type import RagasEvalTypeId
from athina.metrics.metric_type import MetricType
from ragas.metrics.critique import conciseness
from typing import List, Optional

"""
RAGAS Conciseness Docs: https://docs.ragas.io/en/latest/concepts/metrics/critique.html
RAGAS Conciseness Github: https://github.com/explodinggradients/ragas/blob/main/src/ragas/metrics/critique.py
"""
class RagasConciseness(RagasEvaluator):
    """
    This evaluates if the generated llm response conveys information or ideas clearly and efficiently, without unnecessary or redundant details
    """
    @property
    def name(self):
        return RagasEvalTypeId.RAGAS_CONCISENESS.value

    @property
    def display_name(self):
        return "Ragas Conciseness"

    @property
    def metric_ids(self) -> List[str]:
        return [MetricType.RAGAS_CONCISENESS.value]
    
    @property
    def ragas_metric(self):
        return conciseness
    
    @property
    def ragas_metric_name(self):
        return "conciseness"

    @property
    def default_model(self):
        return Model.GPT35_TURBO.value

    @property
    def required_args(self):
        return ["query", "context", "response"]

    @property
    def examples(self):
        return None
    
    @property
    def grade_reason(self) -> str:
        return "This is calculated by how efficiently generated llm response conveys information or ideas clearly and efficiently, without unnecessary or redundant details"

    def is_failure(self, score) -> Optional[bool]:
        return bool(score < self._failure_threshold) if self._failure_threshold is not None else None
        
    def generate_data_to_evaluate(self, context, query, response, **kwargs) -> dict:
        """
        Generates data for evaluation.

        :param context: list of strings of retrieved context
        :param query: user query
        :param response: llm response
        :return: A dictionary with formatted data for evaluation.
        """
        data = {
            "contexts": [context],
            "question": [query],
            "answer": [response],
        }
        return data



from typing import Callable, Any, Dict, Optional

from athina.steps.base import Step


class Assert(Step):
    """
    Step that asserts a condition and raises an error if it fails.

    Attributes:
        condition (Callable[[Any, Dict[str, Any]], bool]): Function to evaluate the assertion.
        error_message (str): Error message to raise if the assertion fails.
    """

    condition: Callable[[Any], bool]
    error_message: str

    def execute(self, input_data: Any) -> Any:
        """Assert a condition and raise error if it fails."""
        if not self.condition(input_data):
            raise AssertionError(f"Assertion failed - {self.error_message}")
        return input_data


class If(Step):
    """
    Step that conditionally runs either the 'then' or 'else' step based on a condition.

    Attributes:
        condition (Callable[[Any], bool]): Function to evaluate the condition.
        then_step (Step): Step to run if the condition is true.
        else_step (Optional[Step]): Step to run if the condition is false.
    """

    condition: Callable[[Any], bool]
    then_step: Step
    else_step: Optional[Step] = None

    def execute(self, input_data: Any) -> Any:
        """Run the 'then' step if the condition is true, else run the 'else' step."""
        result = (
            self.then_step.execute(input_data)
            if self.condition(input_data)
            else self.else_step.execute(input_data) if self.else_step else None
        )
        return result



# Step to extract entities from text using the instructions.
from typing import Union, Dict, Any
from athina.steps import Step
import marvin


class ExtractEntities(Step):
    """
    Step that extracts entities from text using the instructions provided to the step.

    Attributes:
        input_column: The row's column to extract entities from.
        instructions: The instructions to extract entities from the text.
        llm_api_key: The API key for the language model.
        language_model_id: The language model ID to use for entity extraction.
    """

    input_column: str
    instructions: str
    llm_api_key: str
    language_model_id: str

    def execute(self, input_data: Any) -> Union[Dict[str, Any], None]:
        """Extract entities from the text and return the entities."""

        if input_data is None:
            input_data = {}

        if not isinstance(input_data, dict):
            raise TypeError("Input data must be a dictionary.")

        input_text = input_data.get(self.input_column, None)
        
        if input_text is None:
            return None
        
        marvin.settings.openai.api_key = self.llm_api_key
        marvin.settings.openai.chat.completions.model = self.language_model_id

        result = marvin.extract(
            input_text,
            instructions=self.instructions,
        )
        return result    



import time
import traceback
from typing import List, Optional
from athina.interfaces.model import Model
from athina.interfaces.result import EvalResult
from athina.llms.abstract_llm_service import AbstractLlmService
from athina.loaders.summary_loader import SummaryDataPoint
from athina.metrics.metric_type import MetricType
from ..llm_evaluator import LlmEvaluator
from athina.evals.eval_type import LlmEvalTypeId
from athina.llms.question_answerer import QuestionAnswerer
from athina.llms.question_answerer_bulk import QuestionAnswererBulk
from athina.llms.question_generator import QuestionGenerator
from athina.interfaces.result import EvalResultMetric

class SummaryAccuracy(LlmEvaluator):
    """
    This evaluator can be configured with custom examples and instructions.
    """
    questions: List[str] = []
    _llm_service: AbstractLlmService
    _agreement_score_failure_threshold: Optional[float] = None
    _contradiction_score_failure_threshold: Optional[float] = None
    _hallucination_score_failure_threshold: Optional[float] = None

    def __init__(
        self,
        questions: Optional[List[str]] = None,
        n_questions: int = 10,
        model: str = "gpt-4-1106-preview",
        question_answerer: Optional[QuestionAnswerer] = None,
        llm_service: Optional[AbstractLlmService] = None,
        agreement_score_failure_threshold: Optional[float] = None,
        contradiction_score_failure_threshold: Optional[float] = None,
        hallucination_score_failure_threshold: Optional[float] = None,
    ):
        """
        Initialize the evaluator with given parameters.

        Args:
        - loader: An instance of SummarizationLoader.
        - n_questions: Number of questions to generate for summaries.
        - llm_model: Language model to be used.
        - metrics: List of metrics for evaluation.
        """

        # Intialize LLMs
        self._model = model
        self.n_questions = n_questions
        if questions is not None:
            self.questions = questions
        self.question_generator = QuestionGenerator(
            self._model,
            n_questions,
            llm_service=llm_service
        )
        if question_answerer is None:
            self.question_answerer = QuestionAnswererBulk(
                model=self._model,
                llm_service=llm_service
            )
        else:
            self.question_answerer = question_answerer
        self.n_instances = 0
        self.label_counts = {}
        for metric in self.metric_ids:
            setattr(self, f"{metric}_scores", {})

        if agreement_score_failure_threshold is not None:
            self._agreement_score_failure_threshold = agreement_score_failure_threshold
        if hallucination_score_failure_threshold is not None:
            self._hallucination_score_failure_threshold = hallucination_score_failure_threshold
        if contradiction_score_failure_threshold is not None:
            self._contradiction_score_failure_threshold = contradiction_score_failure_threshold

    @property
    def name(self):
        return LlmEvalTypeId.SUMMARIZATION_HAL.value
        
    @property
    def metric_ids(self) -> List[str]:
        return [
            MetricType.AGREEMENT_SCORE.value,
            MetricType.CONTRADICTION_SCORE.value,
            MetricType.HALLUCINATION_SCORE.value
        ]
        
    @property
    def display_name(self):
        return "Summary Accuracy"
        
    @property
    def default_model(self):
        return Model.GPT35_TURBO.value
        
    @property
    def required_args(self):
        return ["document", "response"]
        
    @property
    def examples(self):
        return []
    
    def reason(self) -> str:
        disagreement_answers = self._disagreement_answers()
        if len(disagreement_answers) == 0:
            return "No disagreement between document and summary."
        reason_str = ""
        for question, answer_doc, answer_sum in disagreement_answers:
            reason_str += f"{question}\n- Document: {answer_doc}\n- Summary: {answer_sum}\n"
        return reason_str


    def is_failure(self, metrics) -> Optional[bool]:
        if (self._agreement_score_failure_threshold is None and
            self._contradiction_score_failure_threshold is None and
            self._hallucination_score_failure_threshold is None):
            return None

        threshold_mapping = {
            MetricType.AGREEMENT_SCORE.value: self._agreement_score_failure_threshold,
            MetricType.CONTRADICTION_SCORE.value: self._contradiction_score_failure_threshold,
            MetricType.HALLUCINATION_SCORE.value: self._hallucination_score_failure_threshold,
        }

        for metric in metrics:
            failure_threshold = threshold_mapping.get(metric['id'], None)
            print("failure_threshold", failure_threshold)
            if failure_threshold is not None:
                if metric['id'] == MetricType.AGREEMENT_SCORE.value:
                    if metric['value'] < failure_threshold:  # Fail if agreement score is below its threshold
                        return True
                else:  # For CONTRADICTION_SCORE and HALLUCINATION_SCORE
                    if metric['value'] > failure_threshold:  # Fail if contradiction or hallucination score is above its threshold
                        return True

        return False  # No failure detected 

    def _evaluate(self, **instance) -> EvalResult:
        """
        Run the LLM evaluator.
        """
        start_time = time.time()

        # Validate that correct args were passed
        self.validate_args(**instance)

        summary_datapoint = SummaryDataPoint(**instance)

        # Run the Summary Accuracy evaluator
        summary_eval_result = self._evaluate_element(summary_datapoint)

        end_time = time.time()
        eval_runtime_ms = int((end_time - start_time) * 1000)
        
        metrics = [EvalResultMetric(id=metric_id, value=summary_eval_result[metric_id]) for metric_id in self.metric_ids]
        

        llm_eval_result = EvalResult(
            name=self.name,
            display_name=self.display_name,
            data=SummaryDataPoint(**instance),
            failure=self.is_failure(metrics=metrics),
            reason=self.reason(),
            runtime=eval_runtime_ms,
            model=self._model,
            metrics=metrics,
        )

        return {k: v for k, v in llm_eval_result.items() if v is not None}
    
    def _disagreement_answers(self):
        """Return the questions for which the Y/N answers do not match between document and summary."""
        disagreement_answers = []
        for question in self.answers_doc:
            answer_doc = self.answers_doc[question]
            answer_sum = self.answers_sum[question]
            if answer_doc != answer_sum:
                disagreement_answers.append((question, answer_doc, answer_sum))
        return disagreement_answers

    def _evaluate_element(self, instance: SummaryDataPoint):
        """Evaluate an instance for hallucination."""
        try:
            # Parse instance
            document = instance["document"]
            summary = instance["response"]
            if "label" in instance:
                label = instance["label"]
            else:
                label = "overall"
        except Exception as e:
            print("Exception while parsing instance", e)
            traceback.print_exc()
            raise e

        try:
            # Generate questions based on summary
            if self.questions is None or len(self.questions) == 0:
                self.questions = self.question_generator.generate(summary)

            self.answers_doc = self.question_answerer.answer(questions=self.questions, context=document)[1]
            self.answers_sum = self.question_answerer.answer(questions=self.questions, context=summary)[1]
            metric_results = {}
        except Exception as e:
            print("Exception while generating answers", e)
            traceback.print_exc()
            raise e

        try:
            # Compute metrics
            if self.answers_doc is None or self.answers_sum is None or self.questions is None:
                raise Exception("Validation error - unable to generate answers")
            else:
                for metric in self.metric_ids:
                    metric_name = metric
                    metric_class = MetricType.get_class(metric)
                    metric_result, explanation = metric_class.compute(
                        self.answers_doc, self.answers_sum, self.questions, self.n_questions
                    )
                    metric_results[metric_name] = metric_result
                    metric_results[f"reason_{metric_name}"] = explanation
                    self.update_metric_aggregated_score(metric_name, label, metric_result)
                self.n_instances = self.n_instances + 1
                self.label_counts[label] = self.label_counts.get(label, 0) + 1
            return {
                "questions": self.questions,
                "answers_doc": self.answers_doc,
                "answers_sum": self.answers_sum,
                "label": label,
                **metric_results,
            }
        except Exception as e:
            print("Exception while computing metrics", e)
            traceback.print_exc()
            raise e

    def update_metric_aggregated_score(self, metric, label, aggr_score):
        """Update the aggregated score for a specific metric and label."""
        metric_aggregated_scores = getattr(self, f"{metric}_scores", {})
        current_score = metric_aggregated_scores.get(label, 0)
        metric_aggregated_scores[label] = current_score + aggr_score
        setattr(self, f"{metric}_scores", metric_aggregated_scores)

    def get_metric_aggr(self, metric, label):
        """Compute the average scores based on the provided score dictionary."""
        metric_aggr = getattr(self, f"{metric}_scores", {})
        return metric_aggr.get(label, None)

    def get_average_scores(self, score_dict):
        """Compute average scores for a metric"""
        avg_scores = {}
        sum_score = 0
        n_instances = 0
        for label_type, total_score in score_dict.items():
            avg_scores[label_type] = total_score / self.label_counts[label_type]
            sum_score = sum_score + total_score
            n_instances = n_instances + self.label_counts[label_type]
        avg_scores["overall"] = sum_score / n_instances
        return avg_scores

    def compute_average_scores(self):
        """Compute average scores for each metric."""
        avg_scores = {}
        for metric in self.metric_ids:
            scores = getattr(self, f"{metric}_scores")
            avg_score = self.get_average_scores(scores)
            avg_scores[metric] = avg_score
        return avg_scores

    


from enum import Enum

from athina.metrics.groundedness import GroundednessScore
from .agreement_score import AgreementScore
from .hallucination_score import HallucinationScore
from .contradiction_score import ContradictionScore
from .ragas_metric import RagasMetric
from .passed import Passed
from .similarity_score import SimilarityScore
from .metric import Metric


class MetricType(Enum):
    AGREEMENT_SCORE = "agreement_score"
    HALLUCINATION_SCORE = "hallucination_score"
    CONTRADICTION_SCORE = "contradiction_score"
    RAGAS_CONTEXT_RELEVANCY = "ragas_context_relevancy"
    RAGAS_CONTEXT_PRECISION = "ragas_context_precision"
    RAGAS_ANSWER_RELEVANCY = "ragas_answer_relevancy"
    RAGAS_FAITHFULNESS = "ragas_faithfulness"
    RAGAS_HARMFULNESS = "ragas_harmfulness"
    RAGAS_MALICIOUSNESS = "ragas_maliciousness"
    RAGAS_COHERENCE = "ragas_coherence"
    RAGAS_CONCISENESS = "ragas_conciseness"
    RAGAS_CONTEXT_RECALL = "ragas_context_recall"
    RAGAS_ANSWER_SEMANTIC_SIMILARITY = "ragas_answer_semantic_similarity"
    RAGAS_ANSWER_CORRECTNESS = "ragas_answer_correctness"
    GROUNDEDNESS = "groundedness"
    PASSED = "passed"
    SIMILARITY_SCORE = "similarity_score"

    # Conversation Metrics
    CONVERSATION_RESOLUTION = "conversation_resolution"
    CONVERSATION_COHERENCE = "conversation_coherence"

    @staticmethod
    def get_class(metric_type):
        """
        Returns the class of the metric type.
        """
        if metric_type == MetricType.AGREEMENT_SCORE.value:
            return AgreementScore
        if metric_type == MetricType.GROUNDEDNESS.value:
            return GroundednessScore
        elif metric_type == MetricType.HALLUCINATION_SCORE.value:
            return HallucinationScore
        elif metric_type == MetricType.CONTRADICTION_SCORE.value:
            return ContradictionScore
        elif (
            metric_type == MetricType.RAGAS_CONTEXT_RELEVANCY.value
            or metric_type == MetricType.RAGAS_CONTEXT_PRECISION.value
            or metric_type == MetricType.RAGAS_ANSWER_RELEVANCY.value
            or metric_type == MetricType.RAGAS_FAITHFULNESS.value
            or metric_type == MetricType.RAGAS_CONTEXT_RECALL.value
            or metric_type == MetricType.RAGAS_ANSWER_SEMANTIC_SIMILARITY.value
            or metric_type == MetricType.RAGAS_ANSWER_CORRECTNESS.value
            or metric_type == MetricType.RAGAS_HARMFULNESS.value
            or metric_type == MetricType.RAGAS_COHERENCE.value
        ):
            return RagasMetric
        elif metric_type == MetricType.PASSED.value:
            return Passed
        elif metric_type == MetricType.SIMILARITY_SCORE.value:
            return SimilarityScore
        elif metric_type == MetricType.CONVERSATION_RESOLUTION.value:
            raise NotImplementedError(
                f"Metric class is not implemented for {metric_type}"
            )
        else:
            raise NotImplementedError(f"Metric type {metric_type} not implemented.")



# athina/evals/__init__.py
from athina.evals.llm.does_response_answer_query.evaluator import (
    DoesResponseAnswerQuery,
)
from athina.evals.llm.context_contains_enough_information.evaluator import (
    ContextContainsEnoughInformation,
)
from athina.evals.base_evaluator import BaseEvaluator
from athina.evals.llm.faithfulness.evaluator import Faithfulness
from athina.evals.llm.grading_criteria.evaluator import GradingCriteria
from athina.evals.llm.custom_prompt.evaluator import CustomPrompt
from athina.evals.llm.summary_accuracy.evaluator import SummaryAccuracy
from athina.evals.llm.groundedness.evaluator import Groundedness
from athina.evals.ragas.context_relevancy.evaluator import RagasContextRelevancy
from athina.evals.ragas.answer_relevancy.evaluator import RagasAnswerRelevancy
from athina.evals.ragas.context_precision.evaluator import RagasContextPrecision
from athina.evals.ragas.faithfulness.evaluator import RagasFaithfulness
from athina.evals.ragas.context_recall.evaluator import RagasContextRecall
from athina.evals.ragas.answer_semantic_similarity.evaluator import (
    RagasAnswerSemanticSimilarity,
)
from athina.evals.ragas.answer_correctness.evaluator import RagasAnswerCorrectness
from athina.evals.ragas.harmfulness.evaluator import RagasHarmfulness
from athina.evals.ragas.maliciousness.evaluator import RagasMaliciousness
from athina.evals.ragas.coherence.evaluator import RagasCoherence
from athina.evals.ragas.conciseness.evaluator import RagasConciseness
from athina.evals.function.function_evaluator import FunctionEvaluator
from athina.evals.llm.llm_evaluator import LlmEvaluator
from athina.evals.grounded.grounded_evaluator import GroundedEvaluator
from athina.evals.safety.pii_detection.evaluator import PiiDetection
from athina.evals.safety.prompt_injection.evaluator import PromptInjection
from athina.evals.safety.content_moderation.evaluator import OpenAiContentModeration

from athina.evals.function.wrapper import (
    ContainsAny,
    Regex,
    ContainsAll,
    Contains,
    ContainsNone,
    ContainsJson,
    ContainsEmail,
    IsJson,
    IsEmail,
    NoInvalidLinks,
    ContainsLink,
    ContainsValidLink,
    Equals,
    StartsWith,
    EndsWith,
    LengthLessThan,
    LengthGreaterThan,
    LengthBetween,
    ApiCall,
    OneLine,
    JsonEval
)
from athina.evals.grounded.wrapper import AnswerSimilarity, ContextSimilarity
from athina.evals.guardrails.gibberish_text.evaluator import NotGibberishText
from athina.evals.guardrails.sfw.evaluator import SafeForWorkText
from athina.evals.guardrails.sensitive_topics.evaluator import ContainsNoSensitiveTopics
from athina.evals.guardrails.profanity_free.evaluator import ProfanityFree
from athina.evals.guardrails.detect_pii.evaluator import DetectPII
from athina.evals.guardrails.reading_time.evaluator import ReadingTime
from athina.evals.guardrails.toxic_language.evaluator import ToxicLanguage
from athina.evals.guardrails.correct_language.evaluator import CorrectLanguage
from athina.evals.guardrails.no_secrets_present.evaluator import NoSecretsPresent
from athina.evals.guardrails.restrict_to_topic.evaluator import RestrictToTopic
from athina.evals.guardrails.unusual_prompt.evaluator import NotUnusualPrompt
from athina.evals.guardrails.politeness_check.evaluator import PolitenessCheck

from athina.evals.conversation.conversation_resolution.evaluator import (
    ConversationResolution,
)

from athina.evals.conversation.conversation_resolution.evaluator import (
    ConversationResolution,
)
from athina.evals.conversation.conversation_coherence.evaluator import (
    ConversationCoherence,
)

__all__ = [
    "BaseEvaluator",
    "LlmEvaluator",
    "DoesResponseAnswerQuery",
    "SummaryAccuracy",
    "ContextContainsEnoughInformation",
    "Faithfulness",
    "RagasContextRelevancy",
    "RagasAnswerRelevancy",
    "RagasContextPrecision",
    "RagasFaithfulness",
    "RagasContextRecall",
    "RagasAnswerSemanticSimilarity",
    "RagasAnswerCorrectness",
    "RagasHarmfulness",
    "RagasMaliciousness",
    "RagasCoherence",
    "RagasConciseness",
    "FunctionEvaluator",
    "GradingCriteria",
    "Groundedness",
    "CustomPrompt",
    "ContainsAny",
    "Regex",
    "ContainsAll",
    "Contains",
    "ContainsNone",
    "ContainsJson",
    "ContainsEmail",
    "IsJson",
    "IsEmail",
    "NoInvalidLinks",
    "ContainsLink",
    "ContainsValidLink",
    "Equals",
    "StartsWith",
    "EndsWith",
    "LengthLessThan",
    "LengthGreaterThan",
    "LengthBetween",
    "OneLine",
    "ApiCall",
    "GroundedEvaluator",
    "AnswerSimilarity",
    "ContextSimilarity",
    "ConversationResolution",
    "ConversationCoherence",
    "PiiDetection",
    "PromptInjection",
    "NotGibberishText",
    "SafeForWorkText",
    "ContainsNoSensitiveTopics",
    "OpenAiContentModeration",
    "ProfanityFree",
    "ReadingTime",
    "DetectPII",
    "ToxicLanguage",
    "CorrectLanguage",
    "NoSecretsPresent",
    "RestrictToTopic",
    "NotUnusualPrompt",
    "PolitenessCheck",
    "JsonEval"
]


# Guardrails Toxic Language
# https://hub.guardrailsai.com/validator/guardrails/toxic_language

import time
from typing import Dict, List, Optional
from athina.helpers.logger import logger
from ...base_evaluator import BaseEvaluator
from athina.metrics.metric_type import MetricType
from athina.interfaces.result import EvalResult, EvalResultMetric

# Passes when the text is freem from toxicity, fails when the text is toxic.
class ToxicLanguage(BaseEvaluator):

    _validation_method: str
    _threshold: float

    def __init__(
        self,
        validation_method: str = "sentence",
        threshold: float = 0.5,
    ):
        from guardrails.hub import ToxicLanguage as GuardrailsToxicLanguage
        from guardrails import Guard

        self._validation_method = validation_method
        self._threshold = threshold
        # Initialize guard
        self._guard = Guard().use(
            GuardrailsToxicLanguage, threshold=self._threshold, validation_method=self._validation_method, on_fail="noop"
        )

    @property
    def name(self) -> str:
        return "ToxicLanguage"

    @property
    def display_name(self) -> str:
        return "Toxic Language"

    @property
    def metric_ids(self) -> List[str]:
        return [MetricType.PASSED.value]

    @property
    def required_args(self) -> List[str]:
        return ["response"]

    @property
    def examples(self):
        pass

    def to_config(self) -> Optional[Dict]:
        return None

    def is_failure(self, result: bool) -> bool:
        return not(bool(result))

    def _evaluate(self, **kwargs) -> EvalResult:
        """
        Run the Guardrails evaluator.
        """

        start_time = time.time()
        self.validate_args(**kwargs)
        metrics = []
        try:
            text = kwargs["response"]
            # Setup Guard
            guard_result = self._guard.validate(text)
            grade_reason = "Text is toxicity-free" if guard_result.validation_passed else "Text is toxic"
            # Boolean evaluator
            metrics.append(
                EvalResultMetric(
                    id=MetricType.PASSED.value,
                    value=float(guard_result.validation_passed),
                )
            )
        except Exception as e:
            logger.error(f"Error occurred during eval: {e}")
            raise e

        end_time = time.time()
        eval_runtime_ms = int((end_time - start_time) * 1000)
        llm_eval_result = EvalResult(
            name=self.name,
            display_name=self.display_name,
            data=kwargs,
            failure=self.is_failure(guard_result.validation_passed),
            reason=grade_reason,
            runtime=eval_runtime_ms,
            model=None,
            metrics=metrics,
        )
        return {k: v for k, v in llm_eval_result.items() if v is not None}